#include "package.h"
#include "word.h"

/**
 * @brief dumpwordコマンドのオプション
 */
static struct option longopts[] = {
    { "arithmetic", no_argument, NULL, 'a' },
    { "logical", no_argument, NULL, 'l' },
    { "version", no_argument, NULL, 'v' },
    { "help", no_argument, NULL, 'h' },
    { 0, 0, 0, 0 },
};

/**
 * @brief dumpwordコマンドのメイン
 *
 * @return 正常終了時は0、エラー発生時は1
 *
 * @param argc コマンドライン引数の数
 * @param *argv[] コマンドライン引数の配列
 */
int main(int argc, char *argv[])
{
    bool logicalmode = false;    /* レジストリの内容を論理値（0から65535）で表示する場合はtrue */
    int opt;
    WORD word;
    const char *version = PACKAGE_VERSION,  *cmdversion = "dumpword of YACASL2 version %s\n";
    const char *usage = "Usage: %s [-alh] WORD\n";

    while((opt = getopt_long(argc, argv, "alvh", longopts, NULL)) != -1) {
        switch(opt) {
        case 'l':
            logicalmode = true;
            break;
        case 'v':
            fprintf(stdout, cmdversion, version);
            return 0;
        case 'h':
            fprintf(stdout, usage, argv[0]);
            return 0;
        case '?':
            fprintf(stderr, usage, argv[0]);
            exit(1);
        }
    }

    /* エラーの定義 */
    cerr_init();
    addcerrlist_word();

    if(argv[optind] == NULL) {
        fprintf(stderr, usage, argv[0]);
        freecerr();
        exit(1);
    }
    /* WORD値に変換 */
    word = nh2word(argv[optind]);
    if(cerr->num > 0) {
        fprintf(stderr, "Dumpword Error - %d: %s\n", cerr->num, cerr->msg);
        freecerr();
        exit(1);
    }
    fprintf(stdout, "%6s: ", argv[optind]);
    print_dumpword(word, logicalmode);
    fprintf(stdout, "\n");
    freecerr();
    return 0;
}
